#include <string>
#include <utility>
#include <vector>

#include "absl/strings/string_view.h"
#include "glog/logging.h"

namespace hamming_distance {
using dist_data =
    std::vector<std::pair<std::pair<absl::string_view, absl::string_view>,
                          std::vector<int>>>;

// Compare two sequences, ignore any position where at least on of the entry is
// '-', return a vector of positions where the two sequences are different.
inline std::vector<int> CompareSequences(const std::string& seq_1,
                                         const std::string& seq_2,
                                         int* sites_compared = nullptr) {
  std::vector<int> ret;
  int sites_compared_ = 0;
  CHECK_EQ(seq_1.size(), seq_2.size());
  for (size_t i = 0; i < seq_1.size(); ++i) {
    if (seq_1[i] != '-' && seq_2[i] != '-') {
      ++sites_compared_;
      if (seq_1[i] != seq_2[i]) {
        ret.push_back(i);
      }
    }
  }

  if (sites_compared) *sites_compared = sites_compared_;
  return ret;
}

// Compare two sequences, output hamming distance for a sliding window into a
// dense vector. If append_sites_compared is true, the last element in the
// return vector is the number of sites compared for the pair.
std::vector<int> WindowHammingDistance(
    const std::string& seq_1, const std::string& seq_2, const int window_size,
    const int step_size, const bool append_sites_compared = false);

// Compute all window hamming distances for all pairs whose first index in
// between the start and the end indices and whose second index is smaller
// than the first. If append_sites_compared is true, the last element in the
// return vector is the number of sites compared for the pair.
void CompareWorker(const std::vector<std::pair<std::string, std::string>>& data,
                   const int start, const int end, const int window_size,
                   const int step_size, const bool append_sites_compared,
                   dist_data* hamming_dist);

}  // namespace hamming_distance
